"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@openapi-generator-plus/types");
const path_1 = __importDefault(require("path"));
const handlebars_1 = __importDefault(require("handlebars"));
const handlebars_templates_1 = require("@openapi-generator-plus/handlebars-templates");
const java_like_generator_helper_1 = require("@openapi-generator-plus/java-like-generator-helper");
const generator_common_1 = require("@openapi-generator-plus/generator-common");
function escapeString(value) {
    value = value.replace(/\\/g, '\\\\');
    value = value.replace(/'/g, '\\\'');
    value = value.replace(/\n/g, '\\n');
    return value;
}
function computeCustomTemplatesPath(configPath, customTemplatesPath) {
    if (configPath) {
        return path_1.default.resolve(path_1.default.dirname(configPath), customTemplatesPath);
    }
    else {
        return customTemplatesPath;
    }
}
function toSafeTypeForComposing(nativeType) {
    if (/[^a-zA-Z0-9_.]/.test(nativeType)) {
        return `(${nativeType})`;
    }
    else {
        return nativeType;
    }
}
exports.createGenerator = (context) => (Object.assign(Object.assign(Object.assign(Object.assign({}, context.baseGenerator()), generator_common_1.commonGenerator()), java_like_generator_helper_1.javaLikeGenerator()), { generatorType: () => types_1.CodegenGeneratorType.CLIENT, toConstantName: (name) => {
        return generator_common_1.pascalCase(name);
    }, toLiteral: (value, options, state) => {
        if (value === undefined) {
            return state.generator.toDefaultValue(undefined, options, state).literalValue;
        }
        const { type, format, required, propertyType } = options;
        if (propertyType === types_1.CodegenPropertyType.ENUM) {
            return `${options.nativeType.toString()}.${state.generator.toEnumMemberName(value, state)}`;
        }
        switch (type) {
            case 'integer': {
                return `${value}`;
            }
            case 'number': {
                return `${value}`;
            }
            case 'string': {
                if (format === 'binary') {
                    throw new Error(`Cannot format literal for type ${type} format ${format}`);
                }
                else if (format === 'date') {
                    /* The date format should be an ISO date, and the timezone doesn't matter */
                    return `new Date("${value}")`;
                }
                else if (format === 'time') {
                    /* Parse the date at 1/1/1970 with a local time (no trailing Z), so it's parsed in the client's locale */
                    return `new Date("1970-01-01T${value}")`;
                }
                else if (format === 'date-time') {
                    /* The date-time format should be an ISO datetime with an offset timezone */
                    return `new Date("${value}")`;
                }
                else {
                    return `'${escapeString(value)}'`;
                }
            }
            case 'boolean':
                return !required ? `java.lang.Boolean.valueOf(${value})` : `${value}`;
            case 'object':
            case 'file':
                throw new Error(`Cannot format literal for type ${type}`);
        }
        throw new Error(`Unsupported type name: ${type}`);
    }, toNativeType: ({ type, format }) => {
        /* See https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#data-types */
        switch (type) {
            case 'integer': {
                return new context.NativeType('number');
            }
            case 'number': {
                return new context.NativeType('number');
            }
            case 'string': {
                switch (format) {
                    case 'date':
                    case 'time':
                    case 'date-time':
                        /* We don't have a mapping library to convert incoming and outgoing JSON, so the rawType of dates is string */
                        return new context.NativeType('Date', {
                            wireType: 'string',
                        });
                    default:
                        return new context.NativeType('string');
                }
            }
            case 'boolean': {
                return new context.NativeType('boolean');
            }
            case 'file': {
                /* JavaScript does have a File type, but it isn't supported by JSON serialization so we don't have a wireType */
                return new context.NativeType('File', {
                    wireType: null,
                });
            }
        }
        throw new Error(`Unsupported type name: ${type}`);
    }, toNativeObjectType: function ({ modelNames }, state) {
        let modelName = 'Api';
        for (const name of modelNames) {
            modelName += `.${state.generator.toClassName(name, state)}`;
        }
        return new context.NativeType(modelName);
    }, toNativeArrayType: ({ componentNativeType }) => {
        return new context.TransformingNativeType(componentNativeType, (nativeTypeString) => {
            return `${toSafeTypeForComposing(nativeTypeString)}[]`;
        });
    }, toNativeMapType: ({ keyNativeType, componentNativeType }) => {
        return new context.ComposingNativeType([keyNativeType, componentNativeType], (nativeTypeStrings) => {
            return `{ [name: ${nativeTypeStrings[0]}]: ${nativeTypeStrings[1]} }`;
        });
    }, toDefaultValue: (defaultValue, options, state) => {
        if (defaultValue !== undefined) {
            return {
                value: defaultValue,
                literalValue: state.generator.toLiteral(defaultValue, options, state),
            };
        }
        const { propertyType, required } = options;
        if (!required) {
            return { literalValue: 'undefined' };
        }
        switch (propertyType) {
            case types_1.CodegenPropertyType.NUMBER:
                return { value: 0, literalValue: state.generator.toLiteral(0, options, state) };
            case types_1.CodegenPropertyType.BOOLEAN:
                return { value: false, literalValue: 'false' };
            case types_1.CodegenPropertyType.ARRAY:
                return { value: [], literalValue: '[]' };
            case types_1.CodegenPropertyType.MAP:
                return { value: {}, literalValue: '{}' };
            default:
                return { literalValue: 'undefined' };
        }
    }, options: (config) => {
        const npm = config.npm;
        const defaultRelativeSourceOutputPath = npm ? 'src' : '';
        const relativeSourceOutputPath = config.relativeSourceOutputPath !== undefined ? config.relativeSourceOutputPath : defaultRelativeSourceOutputPath;
        const npmConfig = npm ? {
            name: npm.name || 'typescript-fetch-api',
            version: npm.version || '0.0.1',
            repository: npm.repository,
            private: npm.private,
        } : undefined;
        const typescriptOptions = typeof config.typescript === 'object' ? {
            target: config.typescript.target || 'ES5',
            libs: config.typescript.libs || [config.typescript.target || 'ES5', 'DOM'],
        } : (typeof config.typescript === 'boolean' && !config.typescript) ? undefined : {
            target: 'ES5',
            libs: ['ES5', 'DOM'],
        };
        return {
            relativeSourceOutputPath,
            npm: npmConfig,
            typescript: typescriptOptions,
            customTemplatesPath: config.customTemplates && computeCustomTemplatesPath(config.configPath, config.customTemplates),
        };
    }, operationGroupingStrategy: () => {
        return context.operationGroupingStrategies.addToGroupsByTagOrPath;
    }, postProcessModel: (model) => {
        function toDisjunction(references, transform) {
            const result = references.reduce((result, reference) => {
                const r = transform(reference.model.propertyNativeType);
                if (!r) {
                    return result;
                }
                if (result) {
                    return `${result} | ${toSafeTypeForComposing(r)}`;
                }
                else {
                    return toSafeTypeForComposing(r);
                }
            }, '');
            if (result) {
                return result;
            }
            else {
                return undefined;
            }
        }
        /* If this model has a discriminator then we change its propertyNativeType to a disjunction */
        if (model.discriminator) {
            if (model.discriminator.references) {
                model.propertyNativeType.nativeType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.nativeType);
                model.propertyNativeType.wireType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.wireType);
                model.propertyNativeType.literalType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.literalType);
                model.propertyNativeType.concreteType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.concreteType);
                if (model.propertyNativeType.componentType && model.propertyNativeType.componentType !== model.propertyNativeType) {
                    model.propertyNativeType.componentType.nativeType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.componentType ? nativeType.componentType.nativeType : nativeType.nativeType);
                    model.propertyNativeType.componentType.wireType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.componentType ? nativeType.componentType.wireType : nativeType.wireType);
                    model.propertyNativeType.componentType.literalType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.componentType ? nativeType.componentType.literalType : nativeType.literalType);
                    model.propertyNativeType.componentType.concreteType = toDisjunction(model.discriminator.references, (nativeType) => nativeType.componentType ? nativeType.componentType.concreteType : nativeType.concreteType);
                }
            }
        }
    }, watchPaths: (config) => {
        const result = [path_1.default.resolve(__dirname, '..', 'templates')];
        if (config.customTemplates) {
            result.push(computeCustomTemplatesPath(config.configPath, config.customTemplates));
        }
        return result;
    }, cleanPathPatterns: () => undefined, exportTemplates: async (outputPath, doc, state) => {
        const hbs = handlebars_1.default.create();
        handlebars_templates_1.registerStandardHelpers(hbs, context, state);
        await handlebars_templates_1.loadTemplates(path_1.default.resolve(__dirname, '..', 'templates'), hbs);
        if (state.options.customTemplatesPath) {
            await handlebars_templates_1.loadTemplates(state.options.customTemplatesPath, hbs);
        }
        const rootContext = {
            generatorClass: '@openapi-generator-plus/typescript-fetch-client-generator',
            generatedDate: new Date().toISOString(),
        };
        const relativeSourceOutputPath = state.options.relativeSourceOutputPath;
        await handlebars_templates_1.emit('api', path_1.default.join(outputPath, relativeSourceOutputPath, 'api.ts'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        await handlebars_templates_1.emit('models', path_1.default.join(outputPath, relativeSourceOutputPath, 'models.ts'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        await handlebars_templates_1.emit('runtime', path_1.default.join(outputPath, relativeSourceOutputPath, 'runtime.ts'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        await handlebars_templates_1.emit('configuration', path_1.default.join(outputPath, relativeSourceOutputPath, 'configuration.ts'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        await handlebars_templates_1.emit('custom.d', path_1.default.join(outputPath, relativeSourceOutputPath, 'custom.d.ts'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        await handlebars_templates_1.emit('index', path_1.default.join(outputPath, relativeSourceOutputPath, 'index.ts'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        if (state.options.npm) {
            await handlebars_templates_1.emit('package', path_1.default.join(outputPath, 'package.json'), Object.assign(Object.assign(Object.assign({}, state.options.npm), state.options), rootContext), true, hbs);
        }
        await handlebars_templates_1.emit('README', path_1.default.join(outputPath, 'README.md'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
        if (state.options.typescript) {
            await handlebars_templates_1.emit('tsconfig', path_1.default.join(outputPath, 'tsconfig.json'), Object.assign(Object.assign(Object.assign({}, state.options.typescript), state.options), rootContext), true, hbs);
        }
        await handlebars_templates_1.emit('gitignore', path_1.default.join(outputPath, '.gitignore'), Object.assign(Object.assign(Object.assign({}, doc), state.options), rootContext), true, hbs);
    } }));
exports.default = exports.createGenerator;
